@@ -1,6 +1,6 @@
 /* 
- * Copyright (C) 2006, Mr Jamie McCracken (jamiemcc@gnome.org)
- * Copyright (C) 2008-2010, Nokia
+ * Copyright (C) 2006, Jamie McCracken <jamiemcc@gnome.org>
+ * Copyright (C) 2008-2010, Nokia <ivan.frade@nokia.com>
  *
  * This library is free software; you can redistribute it and/or
  * modify it under the terms of the GNU Lesser General Public
@@ -692,8 +692,10 @@ tracker_uri_vprintf_escaped (const gchar *format,
 
 	output1 = g_strdup_vprintf (format1->str, args);
 	va_end (args);
-	if (!output1)
+	if (!output1) {
+		va_end (args2);
 		goto cleanup;
+        }
 
 	output2 = g_strdup_vprintf (format2->str, args2);
 	va_end (args2);
@@ -947,18 +949,18 @@ tracker_resources_load (TrackerClient  *client,
  *
  *  /&ast; Create D-Bus connection with no warnings and maximum timeout. &ast;/
  *  client = tracker_client_new (0, G_MAXINT);
- *  query = "SELECT"
+ *  query = "SELECT {"
  *          "  ?album"
  *          "  ?title"
  *          "  COUNT(?song) AS songs"
  *          "  SUM(?length) AS totallength"
- *          "WHERE {"
+ *          "} WHERE {"
  *          "  ?album a nmm:MusicAlbum ;"
  *          "  nie:title ?title ."
  *          "  ?song nmm:musicAlbum ?album ;"
  *          "  nfo:duration ?length"
  *          "} "
- *          "GROUP BY ?album");
+ *          "GROUP BY (?album");
  *
  *  array = tracker_resources_sparql_query (client, query, &error);
  *
