@@ -1325,13 +1325,35 @@ cache_set_metadata_decomposed (TrackerProperty  *property,
 		g_value_unset (&gvalue);
 	} else if (!multiple_values && old_values->n_values > 1) {
 		/* trying to add second value to single valued property */
+		GValue old_value = { 0 };
+		GValue new_value = { 0 };
+		const gchar *old_value_str = NULL;
+		const gchar *new_value_str = NULL;
 
-		g_value_unset (&gvalue);
+		g_value_init (&old_value, G_TYPE_STRING);
+		g_value_init (&new_value, G_TYPE_STRING);
+
+		/* Get both old and new values as strings letting glib do
+		 * whatever transformation needed */
+		if (g_value_transform (g_value_array_get_nth (old_values, 0), &old_value)) {
+			old_value_str = g_value_get_string (&old_value);
+		}
+		if (g_value_transform (g_value_array_get_nth (old_values, 1), &new_value)) {
+			new_value_str = g_value_get_string (&new_value);
+		}
 
 		g_set_error (error, TRACKER_DATA_ERROR, TRACKER_DATA_ERROR_CONSTRAINT,
-		             "Unable to insert multiple values for subject `%s' and single valued property `%s'",
+		             "Unable to insert multiple values for subject `%s' and single valued property `%s' "
+		             "(old_value: '%s', new value: '%s')",
 		             resource_buffer->subject,
-		             field_name);
+		             field_name,
+		             old_value_str ? old_value_str : "<untransformable>",
+		             new_value_str ? new_value_str : "<untransformable>");
+
+		g_value_unset (&old_value);
+		g_value_unset (&new_value);
+		g_value_unset (&gvalue);
+
 	} else {
 		cache_insert_value (table_name, field_name, &gvalue,
 		                    graph != NULL ? ensure_resource_id (graph, NULL) : graph_id,
@@ -2355,7 +2377,7 @@ free_queued_statement (QueuedStatement *queued)
 }
 
 static GList*
-queue_statement (GList *queue, 
+queue_statement (GList *queue,
                  const gchar *graph,
                  const gchar *subject,
                  const gchar *predicate,
@@ -2376,11 +2398,15 @@ queue_statement (GList *queue,
 }
 
 static void
-ontology_transaction_end (GList *ontology_queue)
+ontology_transaction_end (GList *ontology_queue,
+                          GPtrArray *seen_classes,
+                          GPtrArray *seen_properties)
 {
 	GList *l;
 	const gchar *ontology_uri = NULL;
 
+	tracker_data_ontology_process_changes (seen_classes, seen_properties);
+
 	/* Perform ALTER-TABLE and CREATE-TABLE calls for all that are is_new */
 	tracker_data_ontology_import_into_db (TRUE);
 
@@ -2395,10 +2421,10 @@ ontology_transaction_end (GList *ontology_queue)
 
 		/* store ontology in database */
 		tracker_data_ontology_process_statement (queued->graph,
-		                                         queued->subject, 
-		                                         queued->predicate, 
-		                                         queued->object, 
-		                                         queued->is_uri, 
+		                                         queued->subject,
+		                                         queued->predicate,
+		                                         queued->object,
+		                                         queued->is_uri,
 		                                         TRUE, TRUE);
 
 	}
@@ -2444,7 +2470,9 @@ ontology_statement_insert (GList       *ontology_queue,
                            GHashTable  *classes,
                            GHashTable  *properties,
                            GHashTable  *id_uri_map,
-                           gboolean     is_uri)
+                           gboolean     is_uri,
+                           GPtrArray   *seen_classes,
+                           GPtrArray   *seen_properties)
 {
 	const gchar *graph, *subject, *predicate;
 
@@ -2458,8 +2486,9 @@ ontology_statement_insert (GList       *ontology_queue,
 	predicate = g_hash_table_lookup (id_uri_map, GINT_TO_POINTER (predicate_id));
 
 	/* load ontology from journal into memory, set all new's is_new to TRUE */
-	tracker_data_ontology_load_statement ("journal", subject_id, subject, predicate, 
-	                                      object, NULL, TRUE, classes, properties);
+	tracker_data_ontology_load_statement ("journal", subject_id, subject, predicate,
+	                                      object, NULL, TRUE, classes, properties,
+	                                      seen_classes, seen_properties);
 
 	/* Queue the statement for processing after ALTER in ontology_transaction_end */
 	ontology_queue = queue_statement (ontology_queue, graph, subject, predicate, object, is_uri);
@@ -2480,6 +2509,8 @@ tracker_data_replay_journal (GHashTable          *classes,
 	gint last_operation_type = 0;
 	gboolean in_ontology = FALSE;
 	GList *ontology_queue = NULL;
+	GPtrArray *seen_classes = NULL;
+	GPtrArray *seen_properties = NULL;
 
 	tracker_data_begin_db_transaction_for_replay (0);
 
@@ -2539,11 +2570,15 @@ tracker_data_replay_journal (GHashTable          *classes,
 			resource_time = tracker_db_journal_reader_get_time ();
 		} else if (type == TRACKER_DB_JOURNAL_END_TRANSACTION) {
 			if (in_ontology) {
-				ontology_transaction_end (ontology_queue);
+				ontology_transaction_end (ontology_queue, seen_classes, seen_properties);
 				g_list_foreach (ontology_queue, (GFunc) free_queued_statement, NULL);
 				g_list_free (ontology_queue);
 				ontology_queue = NULL;
 				in_ontology = FALSE;
+				tracker_data_ontology_free_seen (seen_classes);
+				seen_classes = NULL;
+				tracker_data_ontology_free_seen (seen_properties);
+				seen_properties = NULL;
 			} else {
 				GError *new_error = NULL;
 				tracker_data_update_buffer_might_flush (&new_error);
@@ -2559,6 +2594,12 @@ tracker_data_replay_journal (GHashTable          *classes,
 			tracker_db_journal_reader_get_statement (&graph_id, &subject_id, &predicate_id, &object);
 
 			if (in_ontology) {
+
+				if (!seen_classes)
+					seen_classes = g_ptr_array_new ();
+				if (!seen_properties)
+					seen_properties = g_ptr_array_new ();
+
 				ontology_queue = ontology_statement_insert (ontology_queue,
 				                                            graph_id,
 				                                            subject_id,
@@ -2567,7 +2608,9 @@ tracker_data_replay_journal (GHashTable          *classes,
 				                                            classes,
 				                                            properties,
 				                                            id_uri_map,
-				                                            FALSE);
+				                                            FALSE,
+				                                            seen_classes,
+				                                            seen_properties);
 				continue;
 			}
 
@@ -2606,6 +2649,12 @@ tracker_data_replay_journal (GHashTable          *classes,
 			if (in_ontology) {
 				const gchar *object_n;
 				object_n = g_hash_table_lookup (id_uri_map, GINT_TO_POINTER (object_id));
+
+				if (!seen_classes)
+					seen_classes = g_ptr_array_new ();
+				if (!seen_properties)
+					seen_properties = g_ptr_array_new ();
+
 				ontology_queue = ontology_statement_insert (ontology_queue,
 				                                            graph_id,
 				                                            subject_id,
@@ -2614,7 +2663,9 @@ tracker_data_replay_journal (GHashTable          *classes,
 				                                            classes,
 				                                            properties,
 				                                            id_uri_map,
-				                                            TRUE);
+				                                            TRUE,
+				                                            seen_classes,
+				                                            seen_properties);
 				continue;
 			}
 
